"use strict";

var _react = _interopRequireDefault(require("react"));

var _faker = _interopRequireDefault(require("faker"));

var _testUtilities = require("../../test-utilities");

var _InlineError = require("../InlineError");

var _Select = require("./Select");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var defaultProps = {
  label: 'Country',
  options: [{
    value: 'CA',
    label: 'Canada'
  }, {
    value: 'US',
    label: 'United States'
  }, {
    value: 'UK',
    label: 'United Kingdom'
  }]
};
describe('<Select />', function () {
  it('renders a select and label', function () {
    var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, defaultProps));
    expect(select).toContainReactComponent('select');
    expect(select).toContainReactComponent('label');
  });
  it('renders a select with a pre-selected value when the value prop is provided', function () {
    var value = 'UK';
    var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, Object.assign({}, defaultProps, {
      value: value
    })));
    expect(select).toContainReactComponent('select', {
      value: value
    });
  });
  it('calls the onChange prop with the selected value when the selected value changes', function () {
    var onChangeSpy = jest.fn();
    var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, Object.assign({}, defaultProps, {
      onChange: onChangeSpy
    })));
    select.find('select').trigger('onChange', {
      target: {
        value: 'UK'
      }
    });
    expect(onChangeSpy).toHaveBeenCalledWith('UK');
  });
  it('renders an error message when the error prop is provided', function () {
    var error = 'Select a country';

    var id = _faker["default"].random.alphaNumeric();

    var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, Object.assign({}, defaultProps, {
      id: id,
      error: error
    })));
    expect(select).toContainReactComponent(_InlineError.InlineError, {
      controlID: id,
      children: error
    });
  });
  it('renders a disabled select when the disabled prop is true', function () {
    var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, Object.assign({}, defaultProps, {
      disabled: true
    })));
    expect(select).toContainReactComponent('select', {
      disabled: true
    });
  });
  it('renders a disabled option when the placeholder is provided and selected', function () {
    var placeholder = 'Select a country';
    var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, Object.assign({}, defaultProps, {
      placeholder: placeholder
    })));
    expect(select).toContainReactComponent('option', {
      disabled: true,
      value: _Select.PLACEHOLDER_VALUE,
      children: placeholder
    });
  });
  it('renders a disabled option when the placeholder is provided and not selected', function () {
    var placeholder = 'Select a country';
    var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, Object.assign({}, defaultProps, {
      placeholder: placeholder,
      value: defaultProps.options[0].value
    })));
    expect(select).toContainReactComponent('option', {
      disabled: true,
      value: _Select.PLACEHOLDER_VALUE,
      children: placeholder
    });
  });
  it('renders a hidden and disabled option when the placeholder is provided, selected and similar to the label', function () {
    var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, Object.assign({}, defaultProps, {
      placeholder: defaultProps.label
    })));
    expect(select).toContainReactComponent('option', {
      disabled: true,
      value: _Select.PLACEHOLDER_VALUE,
      hidden: true
    });
    expect(select).not.toContainReactComponent('option', {
      children: defaultProps.label
    });
  });
  it("doesn't render a hidden and disabled option when the placeholder is provided, not selected and similar to the label", function () {
    var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, Object.assign({}, defaultProps, {
      placeholder: defaultProps.label,
      value: defaultProps.options[0].value
    })));
    expect(select).not.toContainReactComponent('option', {
      disabled: true,
      value: _Select.PLACEHOLDER_VALUE,
      hidden: true
    });
  });
  describe('autocomplete', function () {
    it('sets the select autocomplete attribute to "off" when autocomplete is false', function () {
      var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, Object.assign({}, defaultProps, {
        autocomplete: false
      })));
      expect(select).toContainReactComponent('select', {
        autoComplete: 'none'
      });
    });
    it('sets the select autocomplete attribute to "on" when autocomplete is true', function () {
      var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, Object.assign({}, defaultProps, {
        autocomplete: true
      })));
      expect(select).toContainReactComponent('select', {
        autoComplete: 'on'
      });
    });
    it('sets the select autocomplete attribute to a string representation of an autocomplete descriptor', function () {
      var select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_Select.Select, Object.assign({}, defaultProps, {
        autocomplete: {
          group: 'billing',
          field: 'country'
        }
      })));
      expect(select).toContainReactComponent('select', {
        autoComplete: 'billing country'
      });
    });
  });
});