"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const test_utilities_1 = require("../../test-utilities");
const TextField_1 = require("../TextField");
const Labelled_1 = require("../Labelled");
const Autocomplete_1 = require("./Autocomplete");
jest.mock('../../utilities/focus', () => ({
    isFocused: jest.fn(),
}));
const { isFocused } = jest.requireMock('../../utilities/focus');
const defaultProps = {
    id: 'address',
    label: 'Address',
    name: 'address',
    options: ['address 1', 'address 2'],
    value: '',
    title: 'Suggestions',
    ariaLabel: 'Close Predictions',
};
const autocompleteOptions = (options) => {
    return (<Autocomplete_1.AutocompleteOptions>
      {options &&
        options.map((option) => (<Autocomplete_1.AutocompleteOption key={option}>{option}</Autocomplete_1.AutocompleteOption>))}
    </Autocomplete_1.AutocompleteOptions>);
};
describe('<Autocomplete />', () => {
    beforeEach(() => {
        isFocused.mockReset();
        isFocused.mockImplementation(() => true);
    });
    describe('<Labelled />', () => {
        it('renders a <Labelled />', () => {
            const autocomplete = test_utilities_1.mountWithContext(<Autocomplete_1.Autocomplete {...defaultProps} options={[]}/>);
            expect(autocomplete).toContainReactComponent(Labelled_1.Labelled);
        });
    });
    describe('<Field />', () => {
        it('renders a <Field />', () => {
            const autocomplete = test_utilities_1.mountWithContext(<Autocomplete_1.Autocomplete {...defaultProps} options={[]}/>);
            expect(autocomplete).toContainReactComponent(TextField_1.Field, {
                id: 'address',
            });
        });
        it('sets the default aria-controls and aria-expanded properties', () => {
            const autocomplete = test_utilities_1.mountWithContext(<Autocomplete_1.Autocomplete {...defaultProps} options={[]}/>);
            expect(autocomplete).toContainReactComponent(TextField_1.Field, {
                id: 'address',
                ariaControls: 'address-options',
                ariaExpanded: false,
            });
        });
        it('sets the aria-controls, aria-activedescendant and aria-expanded when the input is focused', () => {
            var _a;
            const autocomplete = test_utilities_1.mountWithContext(<Autocomplete_1.Autocomplete {...defaultProps}/>);
            (_a = autocomplete.find(TextField_1.Field)) === null || _a === void 0 ? void 0 : _a.trigger('onFocus');
            expect(autocomplete).toContainReactComponent(TextField_1.Field, {
                id: 'address',
                ariaControls: 'address-options',
                ariaActiveDescendant: 'address-option-0',
                ariaExpanded: true,
            });
        });
        it('calls the onChange callback when the Field onChange is called', () => {
            var _a;
            const onChangeSpy = jest.fn();
            const autocomplete = test_utilities_1.mountWithContext(<Autocomplete_1.Autocomplete {...defaultProps} onChange={onChangeSpy}/>);
            (_a = autocomplete.find(TextField_1.Field)) === null || _a === void 0 ? void 0 : _a.trigger('onInput', 'address');
            expect(onChangeSpy.mock.calls).toMatchObject([['address']]);
            expect(onChangeSpy).toHaveBeenCalledTimes(1);
        });
    });
    describe('<AutocompleteOptions />', () => {
        it('renders when the input is focused', () => {
            var _a;
            const autocomplete = test_utilities_1.mountWithContext(<Autocomplete_1.Autocomplete {...defaultProps}>
          {autocompleteOptions(defaultProps.options)}
        </Autocomplete_1.Autocomplete>);
            (_a = autocomplete.find(TextField_1.Field)) === null || _a === void 0 ? void 0 : _a.trigger('onFocus');
            expect(autocomplete).toContainReactComponent(Autocomplete_1.AutocompleteOptions, {
                children: expect.any(Array),
            });
        });
        it('renders an <ul> with unique id', () => {
            var _a;
            const autocomplete = test_utilities_1.mountWithContext(<Autocomplete_1.Autocomplete {...defaultProps}>
          {autocompleteOptions(defaultProps.options)}
        </Autocomplete_1.Autocomplete>);
            (_a = autocomplete.find(TextField_1.Field)) === null || _a === void 0 ? void 0 : _a.trigger('onFocus');
            expect(autocomplete).toContainReactComponent('ul', {
                id: 'address-options',
            });
        });
        it('is hidden when the close button is clicked', () => {
            var _a, _b;
            const autocomplete = test_utilities_1.mountWithContext(<Autocomplete_1.Autocomplete {...defaultProps}>
          {autocompleteOptions(defaultProps.options)}
        </Autocomplete_1.Autocomplete>);
            (_a = autocomplete.find(TextField_1.Field)) === null || _a === void 0 ? void 0 : _a.trigger('onFocus');
            (_b = autocomplete.find('button')) === null || _b === void 0 ? void 0 : _b.trigger('onClick');
            expect(autocomplete).not.toContainReactComponent(Autocomplete_1.AutocompleteOptions);
        });
    });
    describe('<AutocompleteOption />', () => {
        it('selects the first option when focus is on field', () => {
            var _a;
            const autocomplete = test_utilities_1.mountWithContext(<Autocomplete_1.Autocomplete {...defaultProps}>
          {autocompleteOptions(defaultProps.options)}
        </Autocomplete_1.Autocomplete>);
            (_a = autocomplete.find(TextField_1.Field)) === null || _a === void 0 ? void 0 : _a.trigger('onFocus');
            const [firstOption, secondOption] = autocomplete.findAll(Autocomplete_1.AutocompleteOption);
            expect(firstOption.prop('isSelected')).toStrictEqual(true);
            expect(secondOption.prop('isSelected')).toStrictEqual(false);
        });
        it('renders a <li> with unique id', () => {
            var _a;
            const autocomplete = test_utilities_1.mountWithContext(<Autocomplete_1.Autocomplete {...defaultProps}>
          {autocompleteOptions(defaultProps.options)}
        </Autocomplete_1.Autocomplete>);
            (_a = autocomplete.find(TextField_1.Field)) === null || _a === void 0 ? void 0 : _a.trigger('onFocus');
            const [firstOption, secondOption] = autocomplete.findAll(Autocomplete_1.AutocompleteOption);
            expect(firstOption).toContainReactComponent('li', {
                id: 'address-option-0',
            });
            expect(secondOption).toContainReactComponent('li', {
                id: 'address-option-1',
            });
        });
    });
    describe('<AutocompleteFooter />', () => {
        it('renders when the input is focused', () => {
            var _a;
            const autocomplete = test_utilities_1.mountWithContext(<Autocomplete_1.Autocomplete {...defaultProps}>
          <Autocomplete_1.AutocompleteFooter>This is a test footer</Autocomplete_1.AutocompleteFooter>;
        </Autocomplete_1.Autocomplete>);
            (_a = autocomplete.find(TextField_1.Field)) === null || _a === void 0 ? void 0 : _a.trigger('onFocus');
            expect(autocomplete).toContainReactComponent(Autocomplete_1.AutocompleteFooter, {
                children: 'This is a test footer',
            });
        });
    });
});
